%% Robust Dynamic Energy Use and Climate Change (Coauthors: Xin Li, Borghan Narajabad, and Ted Temzelides)
% -- Exponential Distribution of Gamma --
% For a detail description of the numerical method, please see Appendix 6.2.2 of the paper.

%% define economic parameters
global Ndeg_1 Nnode_1 z_1 Ndeg_2 Nnode_2 z_2 Ndeg_3 Nnode_3 z_3 Ndeg_4 Nnode_4 z_4 T4 TT4 dT4 ddT4 weights4
global dz_1 node_1 amax_1 amin_1 ext_1 extamin_1 extamax_1 dz_2 node_2 amax_2 amin_2 ext_2 extamin_2 extamax_2
global dz_3 node_3 amax_3 amin_3 ext_3 extamin_3 extamax_3 dz_4 node_4 amax_4 amin_4 ext_4 extamin_4 extamax_4
global beta phi_L phi_0 phi theta nu alpha lambda Delta R_stock A2 A3 kappa1 kappa2 kappa3 rho g

beta=0.985^10; %Nordhaus: 0.985^10, Stern: 0.999^10
phi_L=0.2; % Golosov
phi_0=0.392; % Golosov
phi=0.0228; % Golosov
theta=0.3; % capital share, Golosov
nu=0.04; % energy share, Golosov
% over a fairly short period of time, Golosov
R_stock=255; %  oil stock = 253.8
A2=7693; % coal sector
A3=1311; % green sector
kappa1=0.5008; % for oil sector
kappa2=0.08916; % for coal sector
kappa3=1-kappa1-kappa2;
rho=-0.058;
lambda=(2.379e-5/theta/beta)^(-1); % the inverse of the mean of gamma in the approximating model, set according to Golosov
g=(1+0.02)^10-1; % the growth rate of productivity enhanced by population growth. As seen in Model 1.9, it is equivalent
% to assume polulation grows at the rate g.

% Define the switches
I_numerical=1;
I_shape=0;
I_sim=1;


% Define Value function Iteration criterion and number of period for
% simulation
maxit=500;
tol=1e-6;
N_period=30;

%% define robustness parameters
alpha=0.01; % penalty parameter
Delta=theta/(1-beta*theta)/alpha/lambda;
S_thr=1/Delta;

%% define chebyshev nodes along the 1st dimension ( X=sqrt(2)/2*(P+T) )
Nnode_1=6; Ndeg_1=5;  % Ndeg<=Nnode-1 to get orthogonality.
[z_1, T_1, dT_1, ddT_1, weights_1]=Cheb_nodes(Nnode_1,Ndeg_1);

% define chebyshev nodes along the 2nd dimension( Y=sqrt(2)/2*(-P+T) )
Nnode_2=7; Ndeg_2=6;  % Ndeg<=Nnode-1 to get orthogonality.
[z_2, T_2, dT_2, ddT_2, weights_2]=Cheb_nodes(Nnode_2,Ndeg_2);

% define chebyshev nodes along the 3rd dimension R.
Nnode_3=15; Ndeg_3=14;  % Ndeg<=Nnode-1 to get orthogonality.
[z_3, T_3, dT_3, ddT_3, weights_3]=Cheb_nodes(Nnode_3,Ndeg_3);

% define chebyshev nodes along the 4th dimension N, the population.
Nnode_4=10; Ndeg_4=9;  % Ndeg<=Nnode-1 to get orthogonality.
[z_4, T_4, dT_4, ddT_4, weights_4]=Cheb_nodes(Nnode_4,Ndeg_4);

%% Define the overall T4
T4=kron(T_1,kron(T_2,kron(T_3,T_4)));
weights4=kron(weights_1,kron(weights_2,kron(weights_3,weights_4)));

TT4=sparse(diag(1./weights4)); % =T3'*T3;
dT4=[kron(dT_1,kron(T_2,kron(T_3,T_4))); kron(T_1,kron(dT_2,kron(T_3,T_4)));...
    -kron(T_1,kron(T_2,kron(dT_3,T_4))); -kron(T_1,kron(T_2,kron(T_3,dT_4)))];
% the 3rd and 4th dimentional gradients are '-' since value function is
% increasing in R and N
ddT4=[kron(ddT_1,kron(T_2,kron(T_3,T_4))); kron(T_1,kron(ddT_2,kron(T_3,T_4)));...
    kron(T_1,kron(T_2,kron(ddT_3,T_4))); kron(T_1,kron(T_2,kron(T_3,ddT_4)))];
% Four dimentional Hessian, for simplicity, don't calculate the cross derivatives


%% define the grid of points for each dimension (refer to Table 3 in the Appendix 6.2.2 of the paper)
% define the mapping from [-1,1] to state 1 ( X=sqrt(2)/2*(P+T) )
amin_1=0; 
amax_1=sqrt(2)/2*S_thr-0.5; 
[node_1 ext_1 extamin_1 extamax_1 dz_1]=Mapping(amax_1,amin_1,z_1);

% define the mapping from [-1,1] to state 2 ( Y=sqrt(2)/2*(-P+T) )
amin_2=-sqrt(2)/2*S_thr; 
amax_2=sqrt(2)/2*S_thr;
[node_2 ext_2 extamin_2 extamax_2 dz_2]=Mapping(amax_2,amin_2,z_2);

% define the mapping from [-1,1] to state 3 (R: energy stock)
amin_3=0.5; amax_3=R_stock; % extend R_stock for more accuracy.
[node_3 ext_3 extamin_3 extamax_3 dz_3]=Mapping(amax_3,amin_3,z_3);

% define the mapping from [-1,1] to state 4 (N)
amin_4=0.8; amax_4=320; % since g=0.2190 and N_period=30;
[node_4 ext_4 extamin_4 extamax_4 dz_4]=Mapping(amax_4,amin_4,z_4);

%% define test nodes:
multi=2;
test_T_1=Cheb_test_nodes(Nnode_1,Ndeg_1,z_1,multi);
test_T_2=Cheb_test_nodes(Nnode_2,Ndeg_2,z_2,multi);
test_T_3=Cheb_test_nodes(Nnode_3,Ndeg_3,z_3,multi);
test_T_4=Cheb_test_nodes(Nnode_4,Ndeg_4,z_4,multi);
test_T=kron(test_T_1,kron(test_T_2,kron(test_T_3,test_T_4)));


%% main iteration
if I_numerical==1
    % load Mix_Regime.mat
    coef=zeros((Ndeg_1+1)*(Ndeg_2+1)*(Ndeg_3+1)*(Ndeg_4+1),1);
    
    E_opt=zeros(Nnode_1,Nnode_2,Nnode_3,Nnode_4,3);
    test_f=test_T*coef; % the value at the standard node_1(s)
    crit=zeros(maxit,1);
    exitflag=zeros(Nnode_1,Nnode_2,Nnode_3,Nnode_4);
    exit_quad=zeros(maxit,1);
    N_shape=0; % the NO. of shape-preserving
    time=cputime;
    
    matlabpool close force local
    matlabpool open local 7  % requested 7 nodes in the Mix_regime_par.pbs
    
    tic
    for iter=1: maxit
        f=zeros(Nnode_1,Nnode_2,Nnode_3,Nnode_4); % vectorized value function
        test_f_old=test_f; % base on coef in the previous round
        parfor i=1:Nnode_1 % state 1 (P)
            [f(i,:,:,:) E_opt(i,:,:,:,:) exitflag(i,:,:,:)]=Bellman_mix_par(coef,i,beta,phi_L,phi_0,phi,theta,nu,alpha,Delta,g,...
                Ndeg_1,node_1,dz_1,extamin_1,Nnode_2,Ndeg_2,node_2,dz_2,extamin_2,Nnode_3,Ndeg_3,node_3,dz_3,extamin_3,...
                Nnode_4,Ndeg_4,node_4,dz_4,extamin_4, A2,A3,kappa1,kappa2,kappa3,rho,extamax_1,extamax_2);
        end
        f_old=f;
        f=reshape(permute(f,[4 3 2 1]),Nnode_1*Nnode_2*Nnode_3*Nnode_4,1); % vectorized value function
        coef=weights4.*(T4'*f); % update coefficients
        % if the value function is not decreasing or not concave in S, then
        % perform the shape-preserving method
        if I_shape==1 % && iter>40
            if (max(dT4*coef)>0 || max(ddT4*coef)>0)
                [coef exit_quad(iter)]=shape_pre(f,coef);
                N_shape=N_shape+1;
                fprintf('reshaping\n')
            end
        end
        
        test_f=test_T*coef;
        crit(iter)=max(abs(test_f-test_f_old));
        if crit(iter)<tol
            fprintf('Iter=%iter, crit=%1.3e\n',iter,crit(iter));
            break;
        end
        if mod(iter,1)==0
            fprintf('Iter=%iter, crit=%1.3e\n',iter,crit(iter));
        end
    end
    toc
    fprintf('Running Time=%f sec\n',cputime-time);
    v_fn=permute(reshape(f,Nnode_4,Nnode_3,Nnode_2,Nnode_1),[4 3 2 1]);
    save('Mix_regime.mat','iter','N_shape','coef','E_opt','exitflag','v_fn');
    matlabpool close force local
end

%% Simulation
if I_sim==1
%     if I_numerical==0
%         load Coef.mat
%     end
    N_seq=zeros(N_period,1);
    P_seq=zeros(N_period,1);
    T_seq=zeros(N_period,1);
    R_seq=zeros(N_period,1);
    E_seq=zeros(N_period,3);
    X_seq=zeros(N_period,1);
    Y_seq=zeros(N_period,1);
    % initial N, P, T and R
    N_seq(1)=1; % Golosov
    P_seq(1)=103; % Golosov
    T_seq(1)=699-581; % Golosov
    R_seq(1)=253.8; % Golosov
    for t=1:N_period
        X_seq(t)=sqrt(2)/2*(P_seq(t)+T_seq(t));
        Y_seq(t)=sqrt(2)/2*(-P_seq(t)+T_seq(t));
        E_seq(t,:)=Bellman_mix_sim(coef,X_seq(t),Y_seq(t),R_seq(t),N_seq(t)); % x_opt=(E1,E2,E3)
        if t<N_period
            N_seq(t+1)=N_seq(t)*(1+g);
            P_seq(t+1)=P_seq(t)+phi_L*(E_seq(t,1)+E_seq(t,2));
            T_seq(t+1)=(1-phi)*T_seq(t)+(1-phi_L)*phi_0*(E_seq(t,1)+E_seq(t,2));
            R_seq(t+1)=R_seq(t)-E_seq(t,1);
        end
    end
    S_seq=P_seq+T_seq;
    
    df_dX=zeros(N_period,1);
    df_dY=zeros(N_period,1);
    GDP=7e5;
    for i=1:N_period
        df_dX(i)=kron(cheb_D_par(dz_1*(X_seq(i)-extamin_1)-1,Ndeg_1)*dz_1,...
            kron(cheb_par(dz_2*(Y_seq(i)-extamin_2)-1,Ndeg_2),...
            kron(cheb_par(dz_3*(R_seq(i)-extamin_3)-1,Ndeg_3),...
            cheb_par(min(dz_4*(N_seq(i)-extamin_4)-1,1),Ndeg_4))))*coef;
        df_dY(i)=kron(cheb_par(dz_1*(X_seq(i)-extamin_1)-1,Ndeg_1)*dz_1,...
            kron(cheb_D_par(dz_2*(Y_seq(i)-extamin_2)-1,Ndeg_2),...
            kron(cheb_par(dz_3*(R_seq(i)-extamin_3)-1,Ndeg_3),...
            cheb_par(min(dz_4*(N_seq(i)-extamin_4)-1,1),Ndeg_4))))*coef;
    end
    Dam_P=-(1-beta*theta)*sqrt(2)/2*(df_dX-df_dY)*GDP;
    Dam_T=-(1-beta*theta)*sqrt(2)/2*(df_dX+df_dY)*GDP;
    Dam_S=phi_L*Dam_P+(1-phi_L)*phi_0/(1-phi)*Dam_T;
    % save result
    save('Simulation.mat','coef','N_period','N_seq','R_seq','P_seq','T_seq','X_seq','Y_seq','E_seq','S_seq')
end

%%
close all
E_dirty_seq=sum(E_seq(:,[1 2]),2);
n_extrap=N_period+1:N_period+20;
E_dirty_extrap=interp1(1:N_period,E_dirty_seq,n_extrap,'pchip','extrap')';
P_extrap=zeros(length(n_extrap),1);
T_extrap=zeros(length(n_extrap),1);
P_extrap(1)=P_seq(N_period);
T_extrap(1)=T_seq(N_period);
for t=1:length(n_extrap)-1
    P_extrap(t+1)=P_extrap(t)+phi_L*E_dirty_extrap(t);
    T_extrap(t+1)=(1-phi)*T_extrap(t)+(1-phi_L)*phi_0*E_dirty_extrap(t);
end
%%
close gcf
hold on
figure(1)
plot([0,S_thr],[S_thr,0],'k--','linewidth',1.3)
plot(P_seq(1:N_period),T_seq(1:N_period),'k','linewidth',1.3)
scatter(P_seq(1),T_seq(1),'ok')
scatter(P_seq(N_period),T_seq(N_period),'*k')
legend('\Delta^{-1} (breakout point)','Simulated Trajectory of P_t and T_t','t=1','t=30')
xlabel('P_t')
ylabel('T_t')
title('Trajectory of P and T under the Exponential Case with \alpha=0.01')
hold off
saveas(gcf,'Trajectory of P and T.jpeg')


%%
figure(2)
subplot(121)
plot(1:N_period,P_seq(1:N_period),'b',1:N_period,T_seq(1:N_period),'m',1:N_period,S_seq(1:N_period),'k')
xlabel('t')
ylabel('carbon concentration')
title('Simulated path of P, T, and S')
legend('P_t','T_t','S_t','Location','southeast')
axis([0 N_period 0 300])

subplot(122)
plot(1:N_period,Dam_P(1:N_period),'b',1:N_period,Dam_T(1:N_period),'m',1:N_period,Dam_S(1:N_period),'k')
xlabel('t')
ylabel('externality of concentration in dollar')
title('Simulated externality of P, T, and S')
legend('marginal externality of P_t','marginal externality of T_t','marginal externality of S_t','Location','southeast') 
axis([0 N_period 0 7000])
saveas(gcf,'Simulated path of PTS.jpg')










